@@ -2,7 +2,7 @@
 
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 KeyIdentity GmbH
 #
 #    This file is part of LinOTP userid resolvers.
 #
@@ -24,14 +24,15 @@
 #    Contact: www.linotp.org
 #    Support: www.lsexperts.de
 #
-"""This module implements the communication and data mapping to SQL servers.
-   The LinOTP server imports this module to use SQL databases as a userstore.
+"""
+This module implements the communication and data mapping to SQL servers.
+The LinOTP server imports this module to use SQL databases as a userstore.
 
-  Dependencies: UserIdResolver
+Dependencies: UserIdResolver
 """
 
 #from sqlalchemy.event import listen
-import crypt
+
 from sqlalchemy import create_engine
 from sqlalchemy import types
 from sqlalchemy.sql import expression
@@ -39,18 +40,28 @@ from sqlalchemy import Table, MetaData
 from sqlalchemy.orm import sessionmaker
 from sqlalchemy.exc import NoSuchColumnError
 
-from useridresolver.UserIdResolver import UserIdResolver
+from . import resolver_registry
+from useridresolver.UserIdResolver import (UserIdResolver,
+                                           ResolverLoadConfigError
+                                           )
 
 import re
 import base64
 import hashlib
 
-import linotp.lib.phppass as phppass
+import urllib
 
-import sys
-if sys.version_info[0:2] >= (2, 6):
+
+import crypt
+try:
+    import bcrypt
+    _bcrypt_hashpw = bcrypt.hashpw
+except ImportError:
+    _bcrypt_hashpw = None
+
+try:
     import json
-else:
+except:
     import simplejson as json
 
 import traceback
@@ -60,19 +71,46 @@ log = logging.getLogger(__name__)
 DEFAULT_ENCODING = "utf-8"
 
 
+def check_php_password(password, stored_hash):
+    """
+    from phppass: check certain kinds of phppassowrds
+
+    :param password: the new, to be verified password
+    :param stored_hash: the previously used password in a hashed form
+    :return: boolean
+    """
+    result = False 
+
+    if stored_hash.startswith('$2a$'):
+        # bcrypt
+        if _bcrypt_hashpw is None:
+            raise NotImplementedError('The bcrypt module is required')
+        hashed_password = _bcrypt_hashpw(password, stored_hash)
+        result = hashed_password == stored_hash
+    elif stored_hash.startswith('_'):
+        # ext-des
+        hashed_password = crypt.crypt(password, stored_hash)
+        result = hashed_password == stored_hash
+    else:
+        log.info("Hashed password type not recognised!")
+
+    return result
+
+
 def testconnection(params):
-    '''
+    """
     This is used to test if the given parameter set will do a successful
     SQL connection and return the number of found users
     params are:
-        Driver
-        Server
-        Port
-        Database
-        User
-        Password
-        Table
-    '''
+
+    - Driver
+    - Server
+    - Port
+    - Database
+    - User
+    - Password
+    - Table
+    """
     log.debug('[testconnection] %s' % str(params))
 
     num = -1
@@ -85,7 +123,8 @@ def testconnection(params):
                                    params.get("Password"),
                                    params.get("Server"),
                                    params.get("Port"),
-                                   params.get("Database")
+                                   params.get("Database"),
+                                   conParams=params.get('ConnectionParams', "")
                                    )
 
         log.debug("[testconnection] testing connection with connect str: %r"
@@ -95,8 +134,7 @@ def testconnection(params):
         num = dbObj.count(table, params.get("Where", ""))
 
     except Exception as e:
-        log.error('[testconnection] Exception: %r' % e)
-        log.error("[testconnection] %s" % traceback.format_exc())
+        log.exception('[testconnection] Exception: %r' % e)
         err_str = "%r" % e
     finally:
         dbObj.close()
@@ -124,25 +162,47 @@ def make_connect(driver, user, pass_, server, port, db, conParams=""):
     :type     db:     string
     :param    conParams: additional connection parameters
     :type     conParams: string
-
     '''
-    connect = driver + "://"
-    if len(user.strip()) > 0:
-        connect = connect + user
-    if len(pass_.strip()) > 0:
-        connect = connect + ":" + pass_
-    if len(server.strip()) > 0:
-        if len(user.strip()) > 0:
-            connect = connect + "@"
-        connect = connect + server
-    if len(port.strip()) > 0:
-        connect = connect + ":" + str(port)
-    # required db
-    if db != "":
-        connect = connect + "/" + db
 
-    if conParams != "":
-        connect = connect + "?" + conParams
+    connect = ""
+    if "?odbc_connect=" in driver:
+
+        # we have the need to support the odbc_connect mode
+        # where the parameters of the drivers will be concated
+        # The template for the odbc_connect string is submitted
+        # in the field "Additional connection parameters:"
+        param_str = conParams
+        settings = {}
+        settings["{PORT}"] = port
+        settings["{DBUSER}"] = user.strip()
+        settings["{SERVER}"] = server.strip()
+        settings["{PASSWORT}"] = pass_
+        settings["{DATABASE}"] = db
+        for key, value in settings.items():
+            param_str = param_str.replace(key, value)
+
+        url_quote = urllib.quote_plus(param_str)
+        connect = "%s%s" % (driver, url_quote)
+    else:
+        connect = driver + "://"
+        if len(user.strip()) > 0:
+            connect = connect + user
+        if len(pass_.strip()) > 0:
+            connect = connect + ":" + pass_
+        if len(server.strip()) > 0:
+            if len(user.strip()) > 0:
+                connect = connect + "@"
+            connect = connect + server
+        if len(port.strip()) > 0:
+            connect = connect + ":" + str(port)
+        # required db
+        if db != "":
+            connect = connect + "/" + db
+
+        if conParams != "":
+            connect = connect + "?" + conParams
+
+
     return connect
 
 
@@ -213,7 +273,7 @@ def call_on_connect(dbapi_con, connection_record):
     return
 
 
-def _check_hash_type(password, hash_type, hash_value):
+def _check_hash_type(password, hash_type, hash_value, salt=None):
     '''
     Checks, if the password matches the given hash.
 
@@ -241,30 +301,46 @@ def _check_hash_type(password, hash_type, hash_value):
             hashed_password = base64.b64encode(H.digest())
             res = (hashed_password == hash_value)
         except ValueError:
-            log.error("[_check_hash_type] Unsupported Hash type: %r"
+            log.exception("[_check_hash_type] Unsupported Hash type: %r"
                                                                 % hash_type)
-            log.error("[_check_hash_type] %s" % traceback.format_exc())
 
     elif (hash_type.lower()[0:4] == "ssha"):
         log.debug("[_check_hash_type] found a salted hash.")
         try:
             new_hash_type = hash_type[1:]
-            # binary hash value
-            bin_hash = base64.b64decode(hash_value)[:-4]
-            # binary salt
-            bin_salt = base64.b64decode(hash_value)[-4:]
+            # decode the base64 hash value to binary
+            bin_value = base64.b64decode(hash_value)
+
             H = hashlib.new(new_hash_type)
-            H.update(password + bin_salt)
+            hash_len = H.digest_size
+
+            # Carefully check for ASP.NET format (salt separated from hash)
+            if salt and len(bin_value) == hash_len:
+                # Salt separated from hash - probably ASP.NET format
+                # For ASP.NET the format is 'hash(salt + password)' with
+                # 'password' being a UTF-16 little-endian string
+                bin_hash = bin_value
+                password = password.encode('utf-16-le')
+                bin_salt = base64.b64decode(salt)
+                H.update(bin_salt + password)
+            else:
+                # split the hashed password from the binary salt
+                bin_hash = bin_value[:hash_len]
+                bin_salt = bin_value[hash_len:]
+                H.update(password + bin_salt)
+
             bin_hashed_password = H.digest()
             res = (bin_hashed_password == bin_hash)
         except ValueError:
-            log.error("[_check_hash_type] Unsupported Hash type: %r"
+            log.exception("[_check_hash_type] Unsupported Hash type: %r"
                                                                 % hash_type)
-            log.error("[_check_hash_type] %s" % traceback.format_exc())
 
     return res
 
-
+@resolver_registry.class_entry('useridresolver.SQLIdResolver.IdResolver')
+@resolver_registry.class_entry('useridresolveree.SQLIdResolver.IdResolver')
+@resolver_registry.class_entry('useridresolver.sqlresolver')
+@resolver_registry.class_entry('sqlresolver')
 class IdResolver (UserIdResolver):
 
     @classmethod
@@ -322,7 +398,6 @@ class IdResolver (UserIdResolver):
         :return: returns the resolver identifier string
                  or empty string if not exist
         :rtype : string
-
         """
         resolver = "SQLIdResolver.IdResolver"
         if self.conf != "":
@@ -340,6 +415,9 @@ class IdResolver (UserIdResolver):
 
         :return :  true in case of success, false if password does not match
         :rtype :   boolean
+
+        :todo: extend to support htpasswd passwords:
+             http://httpd.apache.org/docs/2.2/misc/password_encryptions.html
         '''
 
         log.info("[checkPass] checking password for user %s" % uid)
@@ -364,12 +442,16 @@ class IdResolver (UserIdResolver):
                 # {SHA256}abcdfef123456
                 hash_type = m.group(1)
                 hash_value = m.group(2)
-                return _check_hash_type(password, hash_type, hash_value)
+
+                # Check for salt field in case the db splits salt from hash:
+                salt = None
+                if 'salt' in userInfo:
+                    salt = userInfo['salt']
+                return _check_hash_type(password, hash_type, hash_value, salt=salt)
             elif m_php:
                 # The Password field contains something like
                 # '$P$BPC00gOTHbTWl6RH6ZyfYVGWkX3Wec.'
-                t_hasher = phppass.PasswordHash(8, False)
-                return t_hasher.check_password(password, userInfo["password"])
+                return check_php_password(password, userInfo["password"])
             else:
                 # get the crypted password and the salt from the database
                 # for doing crypt.crypt( "password", "salt" )
@@ -531,6 +613,9 @@ class IdResolver (UserIdResolver):
         userInfo = userInfo.strip('"')
         try:
             self.sqlUserInfo = json.loads(userInfo)
+        except ValueError as exx:
+            raise ResolverLoadConfigError("Invalid userinfo - no json "
+                                          "document: %s %r" % (userInfo, exx))
         except Exception as  e:
             raise Exception("linotp.sqlresolver.Map: " + str(e))
 
@@ -568,8 +653,7 @@ class IdResolver (UserIdResolver):
             else:
                 log.info('Valid mapping: %r' % self.sqlUserInfo)
         except Exception as e:
-            log.error('[checkMapping] Exception: %s' % (str(e)))
-            log.error("[checkMapping] %s" % traceback.format_exc())
+            log.exception('[checkMapping] Exception: %s' % (str(e)))
 
         log.debug('[checkMapping] done')
         return
@@ -609,8 +693,7 @@ class IdResolver (UserIdResolver):
                 log.info("[getUserId] getting userid %s for user %s" %
                                                         (userId, loginName))
         except Exception as e:
-            log.error('[getUserId] Exception: %s' % (str(e)))
-            log.error("[getUserId] %s" % traceback.format_exc())
+            log.exception('[getUserId] Exception: %s' % (str(e)))
 
         log.debug('[getUserId] done')
         return userId
@@ -640,8 +723,7 @@ class IdResolver (UserIdResolver):
                 userName = row[colName]
 
         except Exception as e:
-            log.error('[getUsername] Exception: %s' % (str(e)))
-            log.error("[getUsername] %s" % traceback.format_exc())
+            log.exception('[getUsername] Exception: %s' % (str(e)))
 
         log.debug('[getUsername] done')
 
@@ -670,8 +752,7 @@ class IdResolver (UserIdResolver):
                 userInfo = self.__getUserInfo(dbObj, row)
 
         except Exception as e:
-            log.error('[getUserInfo] Exception: %s' % (str(e)))
-            log.error("[getUserInfo] %s" % traceback.format_exc())
+            log.exception('[getUserInfo] Exception: %s' % (str(e)))
 
         log.debug('[getUserInfo] done')
         return userInfo
@@ -706,8 +787,7 @@ class IdResolver (UserIdResolver):
                 sf[key] = typ
 
         except Exception as e:
-            log.error('[getSearchFields] Exception: %s' % (str(e)))
-            log.error("[getSearchFields] %s" % traceback.format_exc())
+            log.exception('[getSearchFields] Exception: %s' % (str(e)))
 
         log.debug('[getSearchFields] done')
         return sf
@@ -767,13 +847,11 @@ class IdResolver (UserIdResolver):
                             users[userid] = ui
 
         except KeyError as exx:
-            log.error('[getUserList] Invalid Mapping Error %r' % exx)
-            log.error("[getUserList] %s" % traceback.format_exc())
+            log.exception('[getUserList] Invalid Mapping Error %r' % exx)
             raise KeyError("Invalid Mapping %r " % exx)
 
         except Exception as exx:
-            log.error('[getUserList] Exception: %r' % exx)
-            log.error("[getUserList] %s" % traceback.format_exc())
+            log.exception('[getUserList] Exception: %r' % exx)
 
         log.debug("[getUserList] returning userlist %s" % users.values())
         return users.values()
@@ -837,7 +915,7 @@ class IdResolver (UserIdResolver):
                 value = "-ERR: encoding-"
 
             except NoSuchColumnError as  e:
-                log.error("[__getUserInfo] %s" % traceback.format_exc())
+                log.exception("[__getUserInfo]")
                 value = "-ERR: column mapping-"
 
             userInfo[key] = value
